require "import"
import "android.media.AudioManager"
import "android.content.Context"
import "android.provider.Settings"
import "android.content.ComponentName"
import "android.content.Context"
import "android.content.Intent"
import "android.widget.Button"
import "android.widget.LinearLayout"
import "android.widget.SeekBar"
import "android.widget.TextView"
import "com.androlua.LuaDialog"
import "android.app.NotificationManager"
import "android.os.Build"

context = activity or service
audioManager = context.getSystemService(Context.AUDIO_SERVICE)
notificationManager = context.getSystemService(Context.NOTIFICATION_SERVICE)

function getPhoneMode()
  local mode = audioManager.getRingerMode()
  if mode == AudioManager.RINGER_MODE_SILENT then
    return "Silent"
  elseif mode == AudioManager.RINGER_MODE_VIBRATE then
    return "Vibrate"
  elseif mode == AudioManager.RINGER_MODE_NORMAL then
    return "Ringer"
  else
    return "Unknown"
  end
end

function setVolume(streamType, progress)
  audioManager.setStreamVolume(streamType, progress, 0)
end

function setSoundMode(mode)
  local currentMode = audioManager.getRingerMode()
  if mode == "vibrate" and currentMode ~= AudioManager.RINGER_MODE_VIBRATE then
    audioManager.setRingerMode(AudioManager.RINGER_MODE_VIBRATE)
  elseif mode == "ring" and currentMode ~= AudioManager.RINGER_MODE_NORMAL then
    audioManager.setRingerMode(AudioManager.RINGER_MODE_NORMAL)
  elseif currentMode == AudioManager.RINGER_MODE_SILENT then
    print("The device is currently in silent mode.")
  end
end

function initializeSeekBar(seekBar, streamType)
  local maxVolume = audioManager.getStreamMaxVolume(streamType)
  local currentVolume = audioManager.getStreamVolume(streamType)

  seekBar.setMax(maxVolume)
  seekBar.setProgress(currentVolume)

  seekBar.setOnSeekBarChangeListener(luajava.createProxy("android.widget.SeekBar$OnSeekBarChangeListener", {
    onProgressChanged = function(seekBar, progress, fromUser)
      if fromUser then
        setVolume(streamType, progress)
      end
    end,
    onStartTrackingTouch = function(seekBar) end,
    onStopTrackingTouch = function(seekBar) end,
  }))
end

function togglePhoneMode()
  local currentMode = audioManager.getRingerMode()
  if currentMode == AudioManager.RINGER_MODE_NORMAL then
    setSoundMode("vibrate")
    service.postSpeak(500, "Phone mode: Vibrate")
  elseif currentMode == AudioManager.RINGER_MODE_VIBRATE then
    audioManager.setRingerMode(AudioManager.RINGER_MODE_NORMAL)
service.playSoundTick()
    enableVibration()
    service.postSpeak(500, "Phone mode: Ringer")
  elseif currentMode == AudioManager.RINGER_MODE_SILENT then
    setSoundMode("ring")
service.playSoundTick()
    enableVibration()
    service.postSpeak(500, "Phone mode: Ringer")
  end

  -- تحديث قيم أشرطة التمرير بعد تغيير وضع الصوت
  initializeSeekBar(soundSeekBar, AudioManager.STREAM_RING)
  initializeSeekBar(callSeekBar, AudioManager.STREAM_VOICE_CALL)
  initializeSeekBar(mediaSeekBar, AudioManager.STREAM_MUSIC)
  initializeSeekBar(alarmSeekBar, AudioManager.STREAM_ALARM)
  initializeSeekBar(notificationSeekBar, AudioManager.STREAM_NOTIFICATION)
  initializeSeekBar(accessibilitySeekBar, AudioManager.STREAM_ACCESSIBILITY)
  initializeSeekBar(systemSeekBar, AudioManager.STREAM_SYSTEM)
end

function enableVibration()
  if Build.VERSION.SDK_INT >= Build.VERSION_CODES.M then
    notificationManager.setInterruptionFilter(NotificationManager.INTERRUPTION_FILTER_ALL)
  end
end

layout = {
  LinearLayout,
  layout_width = "match_parent",
  layout_height = "match_parent",
  orientation = 1,

  {
    LinearLayout,
    orientation = 0,
    layout_width = "match_parent",

    {
      TextView,
      id = "textview",
      text = "System audio",
      onClick = function(v)
        dlg.dismiss()
        require 'import'
        import "android.content.*"

        local i = Intent()
        i.setAction(Settings.ACTION_SOUND_SETTINGS)
        this.startActivity(i)
      end,

      onLongClickListener = {
        onLongClick = function(v)
          dlg.dismiss()
          require 'import'
          import "android.content.*"
          local i = Intent()
          i.setComponent(ComponentName("com.android.settings", "com.android.settings.Settings$ZenAccessSettingsActivity"))
          this.startActivity(i)
          return true
        end,
      },
      layout_width = "match_parent",
      layout_height = "wrap_content",
      background = "#ff000000",
      textColor = "#f0f8ff",
      textSize = "25sp",
      gravity = "center",
    },
  },

  {
    LinearLayout,
    orientation = 1,
    layout_width = "match_parent",
    {
      SeekBar,
      id = "soundSeekBar",
      onClick = function(v)
        service.click({{"%speakPhoneMode"}})
        dlg.dismiss()
      end,
      layout_width = "match_parent",
      layout_height = "80dp",
      contentDescription = "ring: ",
      background = "#ff000000",
    },
  },

  {
    LinearLayout,
    orientation = 1,
    layout_width = "match_parent",
    {
      SeekBar,
      id = "callSeekBar",
      onClick = function(v)
        service.click({{"%speakPhoneMode"}})
        dlg.dismiss()
      end,
      layout_width = "match_parent",
      layout_height = "80dp",
      contentDescription = "call: ",
      background = "#ff000000",
    },
  },

  {
    LinearLayout,
    orientation = 1,
    layout_width = "match_parent",
    {
      SeekBar,
      id = "mediaSeekBar",
      onClick = function(v)
        service.click({{"%speakPhoneMode"}})
        dlg.dismiss()
      end,
      layout_width = "match_parent",
      layout_height = "80dp",
      contentDescription = "media: ",
      background = "#ff000000",
    },
  },

  {
    LinearLayout,
    orientation = 1,
    layout_width = "match_parent",
    {
      SeekBar,
      id = "alarmSeekBar",
      onClick = function(v)
        service.click({{"%speakPhoneMode"}})
        dlg.dismiss()
      end,
      layout_width = "match_parent",
      layout_height = "80dp",
      contentDescription = "alarm: ",
      background = "#ff000000",
    },
  },

  {
    LinearLayout,
    orientation = 1,
    layout_width = "match_parent",
    {
      SeekBar,
      id = "notificationSeekBar",
      onClick = function(v)
        service.click({{"%speakPhoneMode"}})
        dlg.dismiss()
      end,
      layout_width = "match_parent",
      layout_height = "80dp",
      contentDescription = "notification: ",
      background = "#ff000000",
    },
  },

  {
    LinearLayout,
    orientation = 1,
    layout_width = "match_parent",
    {
      SeekBar,
      id = "accessibilitySeekBar",
      onClick = function(v)
        service.click({{"%speakPhoneMode"}})
        dlg.dismiss()
      end,
      layout_width = "match_parent",
      layout_height = "80dp",
      contentDescription = "accessibility: ",
      background = "#ff000000",
    },
  },

  {
    LinearLayout,
    orientation = 1,
    layout_width = "match_parent",
    {
      SeekBar,
      id = "systemSeekBar",
      onClick = function(v)
        service.click({{"%speakPhoneMode"}})
        dlg.dismiss()
      end,
      layout_width = "match_parent",
      layout_height = "80dp",
      contentDescription = "system: ",
      background = "#ff000000",
    },
  },

}

function initialize()
  dlg = LuaDialog()
  dlg.setView(loadlayout(layout))
  dlg.setButton("Switch Sound Mode", function ()
    togglePhoneMode()
dlg.dismiss()
initialize()
  end)
  dlg.setButton2("Close.", function ()
    dlg.dismiss()
  end)
  dlg.show()

  initializeSeekBar(soundSeekBar, AudioManager.STREAM_RING)
  initializeSeekBar(callSeekBar, AudioManager.STREAM_VOICE_CALL)
  initializeSeekBar(mediaSeekBar, AudioManager.STREAM_MUSIC)
  initializeSeekBar(alarmSeekBar, AudioManager.STREAM_ALARM)
  initializeSeekBar(notificationSeekBar, AudioManager.STREAM_NOTIFICATION)
  initializeSeekBar(accessibilitySeekBar, AudioManager.STREAM_ACCESSIBILITY)
  initializeSeekBar(systemSeekBar, AudioManager.STREAM_SYSTEM)
end

initialize()